@@ -27,7 +27,7 @@ from xonsh.inspectors import Inspector
 from xonsh.aliases import Aliases, make_default_aliases
 from xonsh.environ import Env, default_env, locate_binary
 from xonsh.jobs import add_job
-from xonsh.platform import ON_POSIX, ON_WINDOWS
+from xonsh.platform import ON_POSIX, ON_WINDOWS, ON_WSL
 from xonsh.proc import (
     PopenThread,
     ProcProxyThread,
@@ -45,7 +45,7 @@ from xonsh.tools import (
     XonshError,
     XonshCalledProcessError,
 )
-from xonsh.lazyimps import pty, termios
+from xonsh.lazyimps import pty, termios, fcntl
 from xonsh.commands_cache import CommandsCache
 from xonsh.events import events
 
@@ -377,18 +377,18 @@ class SubprocSpec:
     executed.
     """
 
-    kwnames = ("stdin", "stdout", "stderr", "universal_newlines")
+    kwnames = ("stdin", "stdout", "stderr", "universal_newlines", "close_fds")
 
     def __init__(
         self,
         cmd,
-        *,
         cls=subprocess.Popen,
         stdin=None,
         stdout=None,
         stderr=None,
         universal_newlines=False,
-        captured=False
+        close_fds=False,
+        captured=False,
     ):
         """
         Parameters
@@ -405,6 +405,9 @@ class SubprocSpec:
             Popen file descriptor or flag for stderr.
         universal_newlines : bool
             Whether or not to use universal newlines.
+        close_fds : bool
+            Whether or not to close the file descriptiors when the
+            process exits.
         captured : bool or str, optional
             The flag for if the subprocess is captured, may be one of:
             False for $[], 'stdout' for $(), 'hiddenobject' for ![], or
@@ -445,6 +448,7 @@ class SubprocSpec:
         self.stdout = stdout
         self.stderr = stderr
         self.universal_newlines = universal_newlines
+        self.close_fds = close_fds
         self.captured = captured
         # pure attrs
         self.args = list(cmd)
@@ -531,6 +535,8 @@ class SubprocSpec:
 
     def run(self, *, pipeline_group=None):
         """Launches the subprocess and returns the object."""
+        event_name = self._cmd_event_name()
+        self._pre_run_event_fire(event_name)
         kwargs = {n: getattr(self, n) for n in self.kwnames}
         self.prep_env(kwargs)
         self.prep_preexec_fn(kwargs, pipeline_group=pipeline_group)
@@ -545,6 +551,7 @@ class SubprocSpec:
         p.last_in_pipeline = self.last_in_pipeline
         p.captured_stdout = self.captured_stdout
         p.captured_stderr = self.captured_stderr
+        self._post_run_event_fire(event_name, p)
         return p
 
     def _run_binary(self, kwargs):
@@ -556,6 +563,11 @@ class SubprocSpec:
             raise XonshError(e.format(self.cmd[0]))
         except FileNotFoundError:
             cmd0 = self.cmd[0]
+            if len(self.cmd) == 1 and cmd0.endswith("?"):
+                with contextlib.suppress(OSError):
+                    return self.cls(
+                        ["man", cmd0.rstrip("?")], bufsize=bufsize, **kwargs
+                    )
             e = "xonsh: subprocess mode: command not found: {0}".format(cmd0)
             env = builtins.__xonsh__.env
             sug = suggest_commands(cmd0, env, builtins.aliases)
@@ -579,7 +591,9 @@ class SubprocSpec:
             return
         if not builtins.__xonsh__.env.get("XONSH_INTERACTIVE"):
             return
-        if pipeline_group is None:
+        if pipeline_group is None or ON_WSL:
+            # If there is no pipeline group
+            # or the platform is windows subsystem for linux (WSL)
             xonsh_preexec_fn = no_pg_xonsh_preexec_fn
         else:
 
@@ -598,6 +612,26 @@ class SubprocSpec:
         for i in range(len(cmd)):
             cmd[i] = cmd[i].replace("\0", "\\0")
 
+    def _cmd_event_name(self):
+        if callable(self.alias):
+            return getattr(self.alias, "__name__", repr(self.alias))
+        elif self.binary_loc is None:
+            return "<not-found>"
+        else:
+            return os.path.basename(self.binary_loc)
+
+    def _pre_run_event_fire(self, name):
+        event_name = "on_pre_spec_run_" + name
+        if events.exists(event_name):
+            event = getattr(events, event_name)
+            event.fire(spec=self)
+
+    def _post_run_event_fire(self, name, proc):
+        event_name = "on_post_spec_run_" + name
+        if events.exists(event_name):
+            event = getattr(events, event_name)
+            event.fire(spec=self, proc=proc)
+
     #
     # Building methods
     #
@@ -742,6 +776,17 @@ def _safe_pipe_properties(fd, use_tty=False):
     props = termios.tcgetattr(fd)
     props[1] = props[1] & (~termios.ONLCR) | termios.ONLRET
     termios.tcsetattr(fd, termios.TCSANOW, props)
+    # newly created PTYs have a stardard size (24x80), set size to the same size
+    # than the current terminal
+    winsize = None
+    if sys.stdin.isatty():
+        winsize = fcntl.ioctl(sys.stdin.fileno(), termios.TIOCGWINSZ, b"0000")
+    elif sys.stdout.isatty():
+        winsize = fcntl.ioctl(sys.stdout.fileno(), termios.TIOCGWINSZ, b"0000")
+    elif sys.stderr.isatty():
+        winsize = fcntl.ioctl(sys.stderr.fileno(), termios.TIOCGWINSZ, b"0000")
+    if winsize is not None:
+        fcntl.ioctl(fd, termios.TIOCSWINSZ, winsize)
 
 
 def _update_last_spec(last):
@@ -902,6 +947,12 @@ def run_subproc(cmds, captured=False):
     if _should_set_title(captured=captured):
         # set title here to get currently executing command
         pause_call_resume(proc, builtins.__xonsh__.shell.settitle)
+    else:
+        # for some reason, some programs are in a stopped state when the flow
+        # reaches this point, hence a SIGCONT should be sent to `proc` to make
+        # sure that the shell doesn't hang. This `pause_call_resume` invocation
+        # does this
+        pause_call_resume(proc, int)
     # create command or return if backgrounding.
     if background:
         return
@@ -1062,6 +1113,8 @@ def convert_macro_arg(raw_arg, kind, glbs, locs, *, name="<arg>", macroname="<ma
         if locs is not None:
             ctx |= set(locs.keys())
         mode = mode or "eval"
+        if mode != "eval" and not raw_arg.endswith("\n"):
+            raw_arg += "\n"
         arg = execer.parse(raw_arg, ctx, mode=mode, filename=filename)
     elif kind is types.CodeType or kind is compile:  # NOQA
         mode = mode or "eval"
@@ -1271,11 +1324,7 @@ def xonsh_builtins(execer=None):
     scope. Likely useful in testing.
     """
     load_builtins(execer=execer)
-    # temporary shims for old __xonsh_*__ builtins
-    load_proxies()
     yield
-    # temporary shims for old __xonsh_*__ builtins
-    unload_proxies()
     unload_builtins()
 
 
@@ -1287,7 +1336,7 @@ class XonshSession:
     def __init__(self, execer=None, ctx=None):
         """
         Parameters
-        ---------
+        ----------
         execer : Execer, optional
             Xonsh execution object, may be None to start
         ctx : Mapping, optional
@@ -1300,7 +1349,7 @@ class XonshSession:
         """Loads the session with default values.
 
         Parameters
-        ---------
+        ----------
         execer : Execer, optional
             Xonsh execution object, may be None to start
         ctx : Mapping, optional
@@ -1353,12 +1402,17 @@ class XonshSession:
 
     def link_builtins(self, execer=None):
         # public built-ins
-        builtins.XonshError = self.builtins.XonshError
-        builtins.XonshCalledProcessError = self.builtins.XonshCalledProcessError
-        builtins.evalx = None if execer is None else execer.eval
-        builtins.execx = None if execer is None else execer.exec
-        builtins.compilex = None if execer is None else execer.compile
-        builtins.events = self.builtins.events
+        proxy_mapping = {
+            "XonshError": "__xonsh__.builtins.XonshError",
+            "XonshCalledProcessError": "__xonsh__.builtins.XonshCalledProcessError",
+            "evalx": "__xonsh__.builtins.evalx",
+            "execx": "__xonsh__.builtins.execx",
+            "compilex": "__xonsh__.builtins.compilex",
+            "events": "__xonsh__.builtins.events",
+        }
+        for refname, objname in proxy_mapping.items():
+            proxy = DynamicAccessProxy(refname, objname)
+            setattr(builtins, refname, proxy)
 
         # sneak the path search functions into the aliases
         # Need this inline/lazy import here since we use locate_binary that
@@ -1376,6 +1430,7 @@ class XonshSession:
             "execx",
             "compilex",
             "default_aliases",
+            "events",
         ]
 
         for name in names:
@@ -1441,167 +1496,5 @@ class DynamicAccessProxy:
     def __call__(self, *args, **kwargs):
         return self.obj.__call__(*args, **kwargs)
 
-
-class DeprecationWarningProxy:
-    """Proxies access, but warns in the process."""
-
-    def __init__(self, oldname, newname):
-        super().__setattr__("oldname", oldname)
-        super().__setattr__("newname", newname)
-
-    @property
-    def obj(self):
-        """Dynamically grabs object"""
-        names = self.newname.split(".")
-        obj = builtins
-        for name in names:
-            obj = getattr(obj, name)
-        return obj
-
-    def warn(self):
-        """Issues deprecation warning."""
-        warnings.warn(
-            "{} has been deprecated, please use {} instead.".format(
-                self.oldname, self.newname
-            ),
-            DeprecationWarning,
-        )
-
-    def __getattr__(self, name):
-        self.warn()
-        return getattr(self.obj, name)
-
-    def __setattr__(self, name, value):
-        self.warn()
-        return super().__setattr__(self.obj, name, value)
-
-    def __delattr__(self, name):
-        self.warn()
-        return delattr(self.obj, name)
-
-    def __getitem__(self, item):
-        self.warn()
-        return self.obj.__getitem__(item)
-
-    def __setitem__(self, item, value):
-        self.warn()
-        return self.obj.__setitem__(item, value)
-
-    def __delitem__(self, item):
-        self.warn()
-        del self.obj[item]
-
-    def __call__(self, *args, **kwargs):
-        self.warn()
-        return self.obj.__call__(*args, **kwargs)
-
-
-def load_proxies():
-    """Loads builtin dynamic access proxies.
-    Also puts temporary shims in place for `__xonsh_*__` builtins.
-    """
-    proxy_mapping = {
-        "XonshError": "__xonsh__.builtins.XonshError",
-        "XonshCalledProcessError": "__xonsh__.builtins.XonshCalledProcessError",
-        "evalx": "__xonsh__.builtins.evalx",
-        "execx": "__xonsh__.builtins.execx",
-        "compilex": "__xonsh__.builtins.compilex",
-        "events": "__xonsh__.builtins.events",
-    }
-    for refname, objname in proxy_mapping.items():
-        proxy = DynamicAccessProxy(refname, objname)
-        setattr(builtins, refname, proxy)
-
-    deprecated_mapping = {
-        "__xonsh_env__": "__xonsh__.env",
-        "__xonsh_history__": "__xonsh__.history",
-        "__xonsh_ctx__": "__xonsh__.ctx",
-        "__xonsh_help__": "__xonsh__.help",
-        "__xonsh_superhelp__": "__xonsh__.superhelp",
-        "__xonsh_pathsearch__": "__xonsh__.pathsearch",
-        "__xonsh_globsearch__": "__xonsh__.globsearch",
-        "__xonsh_regexsearch__": "__xonsh__.regexsearch",
-        "__xonsh_glob__": "__xonsh__.glob",
-        "__xonsh_expand_path__": "__xonsh__.expand_path",
-        "__xonsh_exit__": "__xonsh__.exit",
-        "__xonsh_stdout_uncaptured__": "__xonsh__.stdout_uncaptured",
-        "__xonsh_stderr_uncaptured__": "__xonsh__.stderr_uncaptured",
-        "__xonsh_subproc_captured_stdout__": "__xonsh__.subproc_captured_stdout",
-        "__xonsh_subproc_captured_inject__": "__xonsh__.subproc_captured_inject",
-        "__xonsh_subproc_captured_object__": "__xonsh__.subproc_captured_object",
-        "__xonsh_subproc_captured_hiddenobject__": "__xonsh__.subproc_captured_hiddenobject",
-        "__xonsh_subproc_uncaptured__": "__xonsh__.subproc_uncaptured",
-        "__xonsh_execer__": "__xonsh__.execer",
-        "__xonsh_commands_cache__": "__xonsh__.commands_cache",
-        "__xonsh_all_jobs__": "__xonsh__.all_jobs",
-        "__xonsh_ensure_list_of_strs__": "__xonsh__.ensure_list_of_strs",
-        "__xonsh_list_of_strs_or_callables__": "__xonsh__.list_of_strs_or_callables",
-        "__xonsh_list_of_list_of_strs_outer_product__": "__xonsh__.list_of_list_of_strs_outer_product",
-        "__xonsh_completers__": "__xonsh__.completers",
-        "__xonsh_call_macro__": "__xonsh__.call_macro",
-        "__xonsh_enter_macro__": "__xonsh__.enter_macro",
-        "__xonsh_path_literal__": "__xonsh__.path_literal",
-    }
-    for badname, goodname in deprecated_mapping.items():
-        proxy = DeprecationWarningProxy(badname, goodname)
-        setattr(builtins, badname, proxy)
-
-    if hasattr(builtins.__xonsh__, "pyexit"):
-        builtins.__xonsh_pyexit__ = DeprecationWarningProxy(
-            "builtins.__xonsh_pyexit__", "builtins.__xonsh__.pyexit"
-        )
-    if hasattr(builtins.__xonsh__, "quit"):
-        builtins.__xonsh_pyquit__ = DeprecationWarningProxy(
-            "builtins.__xonsh_pyquit__", "builtins.__xonsh__.pyquit"
-        )
-
-
-def unload_proxies():
-    """Removes the xonsh builtins (proxies) from the Python builtins.
-    """
-    if hasattr(builtins, "__xonsh_pyexit__"):
-        builtins.exit = builtins.__xonsh_pyexit__
-    if hasattr(builtins, "__xonsh_pyquit__"):
-        builtins.quit = builtins.__xonsh_pyquit__
-
-    names = [
-        "__xonsh_env__",
-        "__xonsh_ctx__",
-        "__xonsh_help__",
-        "__xonsh_superhelp__",
-        "__xonsh_pathsearch__",
-        "__xonsh_globsearch__",
-        "__xonsh_regexsearch__",
-        "__xonsh_glob__",
-        "__xonsh_expand_path__",
-        "__xonsh_exit__",
-        "__xonsh_stdout_uncaptured__",
-        "__xonsh_stderr_uncaptured__",
-        "__xonsh_pyexit__",
-        "__xonsh_pyquit__",
-        "__xonsh_subproc_captured_stdout__",
-        "__xonsh_subproc_captured_inject__",
-        "__xonsh_subproc_captured_object__",
-        "__xonsh_subproc_captured_hiddenobject__",
-        "__xonsh_subproc_uncaptured__",
-        "__xonsh_execer__",
-        "__xonsh_commands_cache__",
-        "__xonsh_completers__",
-        "__xonsh_call_macro__",
-        "__xonsh_enter_macro__",
-        "__xonsh_path_literal__",
-        "XonshError",
-        "XonshCalledProcessError",
-        "evalx",
-        "execx",
-        "compilex",
-        "default_aliases",
-        "__xonsh_all_jobs__",
-        "__xonsh_ensure_list_of_strs__",
-        "__xonsh_list_of_strs_or_callables__",
-        "__xonsh_list_of_list_of_strs_outer_product__",
-        "__xonsh_history__",
-    ]
-    for name in names:
-        if hasattr(builtins, name):
-            delattr(builtins, name)
+    def __dir__(self):
+        return self.obj.__dir__()
