@@ -7,16 +7,34 @@ import builtins
 import xonsh.platform as xp
 
 
-def env_name(pre_chars="(", post_chars=")"):
-    """Extract the current environment name from $VIRTUAL_ENV or
-    $CONDA_DEFAULT_ENV if that is set
+def find_env_name():
+    """Finds the current environment name from $VIRTUAL_ENV or
+    $CONDA_DEFAULT_ENV if that is set.
     """
     env_path = builtins.__xonsh__.env.get("VIRTUAL_ENV", "")
     if len(env_path) == 0 and xp.ON_ANACONDA:
         env_path = builtins.__xonsh__.env.get("CONDA_DEFAULT_ENV", "")
     env_name = os.path.basename(env_path)
-    if env_name:
-        return pre_chars + env_name + post_chars
+    return env_name
+
+
+def env_name():
+    """Returns the current env_name if it non-empty, surrounded by the
+    ``{env_prefix}`` and ``{env_postfix}`` fields.
+    """
+    env_name = find_env_name()
+    if builtins.__xonsh__.env.get("VIRTUAL_ENV_DISABLE_PROMPT") or not env_name:
+        # env name prompt printing disabled, or no environment; just return
+        return
+
+    venv_prompt = builtins.__xonsh__.env.get("VIRTUAL_ENV_PROMPT")
+    if venv_prompt is not None:
+        return venv_prompt
+    else:
+        pf = builtins.__xonsh__.shell.prompt_formatter
+        pre = pf._get_field_value("env_prefix")
+        post = pf._get_field_value("env_postfix")
+        return pre + env_name + post
 
 
 def vte_new_tab_cwd():
