@@ -832,12 +832,20 @@ def command_not_found(cmd):
     """
     if not ON_LINUX:
         return ""
-    elif not os.path.isfile("/usr/lib/command-not-found"):
-        # utility is not on PATH
+
+    cnf = builtins.__xonsh__.commands_cache.lazyget(
+        "command-not-found", ("/usr/lib/command-not-found",)
+    )[0]
+
+    if not os.path.isfile(cnf):
         return ""
-    c = "/usr/lib/command-not-found {0}; exit 0"
+
+    c = "{0} {1}; exit 0"
     s = subprocess.check_output(
-        c.format(cmd), universal_newlines=True, stderr=subprocess.STDOUT, shell=True
+        c.format(cnf, cmd),
+        universal_newlines=True,
+        stderr=subprocess.STDOUT,
+        shell=True,
     )
     s = "\n".join(s.rstrip().splitlines()).strip()
     return s
@@ -1548,21 +1556,6 @@ def dict_to_str(x):
     return str(x)
 
 
-def setup_win_unicode_console(enable):
-    """"Enables or disables unicode display on windows."""
-    try:
-        import win_unicode_console
-    except ImportError:
-        win_unicode_console = False
-    enable = to_bool(enable)
-    if ON_WINDOWS and win_unicode_console:
-        if enable:
-            win_unicode_console.enable()
-        else:
-            win_unicode_console.disable()
-    return enable
-
-
 # history validation
 
 _min_to_sec = lambda x: 60.0 * float(x)
